class SneakersyncException(Exception):
    """Base class for sneakersync exceptions."""
    def __init__(self, action, module, text):
        Exception.__init__(self)
        self.action = action
        self.module = module
        self.text = text



import argparse
import logging
import sys

import sneakersync

def main():
    parser = argparse.ArgumentParser(
        description="Send and receive files through the sneakernet")
    parser.add_argument(
        "--verbosity", "-v",
        choices=["error", "warning", "info", "debug"], default="warning")
    
    progress_group = parser.add_mutually_exclusive_group()
    progress_group.add_argument(
        "--progress", action="store_true", default=True, 
        help="Display progress bar (default value, see --no-progress)")
    progress_group.add_argument(
        "--no-progress", dest="progress", action="store_false", 
        help="Display progress bar (see --progress)")
    
    subparsers = parser.add_subparsers(help="Sub-commands help")
    
    send_parser = subparsers.add_parser(
        "send", help="Send data on the sneakernet",
        formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    send_parser.add_argument("destination")
    send_parser.set_defaults(function=sneakersync.send)
    
    receive_parser = subparsers.add_parser(
        "receive", help="Receive data from the sneakernet",
        formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    receive_parser.add_argument("source")
    receive_parser.set_defaults(function=sneakersync.receive)
    
    arguments = vars(parser.parse_args())
    
    verbosity = arguments.pop("verbosity")
    logging.basicConfig(
        level=verbosity.upper(), 
        format="%(levelname)s: %(message)s")
        
    function = arguments.pop("function")
    
    try:
        function(**arguments)
    except sneakersync.SneakersyncException as e:
        sneakersync.logger.error(
            "Could not {} module {}: \n{}".format(
                e.action, e.module["root"], e.text))
        if sneakersync.logger.getEffectiveLevel() <= logging.DEBUG:
            raise
    except Exception as e:
        sneakersync.logger.error(e)
        if sneakersync.logger.getEffectiveLevel() <= logging.DEBUG:
            raise
    
    return 0

if __name__ == "__main__":
    sys.exit(main())



import datetime
import logging
import os
import socket
import sys

import six
import yaml

from .exception import SneakersyncException
import sys
sys.modules["sneakersync"].Exception = SneakersyncException

logger = logging.getLogger(__name__)

from .rsync import *
from .state import State

def send(destination, progress):
    """Send modules on the sneakernet."""
    
    state = State.load(os.path.join(destination, "sneakersync.dat"))
    if state.previous_direction == "send":
        confirmed = confirm(
            "WARNING: "
            "do you want to re-send the current files "
            "(sent from {} on {})?".format(
                state.previous_host, state.previous_date.strftime("%c")))
        if not confirmed:
            return 0
    
    configuration = read_configuration(
        os.path.join(destination, "sneakersync.cfg"))
    
    for module in configuration["modules"]:
        if sneakersync.logger.getEffectiveLevel() <= logging.WARNING:
            print("Sending {}".format(module["root"]))
        
        if not os.path.isdir(module["root"]):
            logger.warn("No such file or directory: {}".format(module["root"]))
            continue
            
        # WARNING: make sure there is no "/" at the end of the module
        module["root"] = module["root"].rstrip("/")
        
        command = [
            "rsync",
            "--archive", "--xattrs", "--delete", "--relative", "--fake-super"
        ]
        command.extend(get_verbosity_options(progress))
        
        command += get_filters(configuration["filters"])
        command += get_filters(module["filters"])
        command += [
            os.path.join("/." + module["root"], ""), 
            os.path.join(destination, "")
        ]
        
        call_subprocess(command, "send", module)
    
    state.previous_direction = "send"
    state.previous_date = datetime.datetime.now()
    state.previous_host = socket.gethostname()
    state.save()

def receive(source, progress):
    """Receive modules from the sneakernet."""
    
    state = State.load(os.path.join(source, "sneakersync.dat"))
    if state.previous_direction == "receive":
        confirmed = confirm(
            "WARNING: "
            "do you want to receive the current files again "
            "(sent from {} on {})?".format(
                state.previous_host, state.previous_date.strftime("%c")))
        if not confirmed:
            return 0
        confirmed = confirm(
            "This will overwrite your data. Are you really sure?")
        if not confirmed:
            return 0
    
    configuration = read_configuration(os.path.join(source, "sneakersync.cfg"))
    
    for module in configuration["modules"]:
        if sneakersync.logger.getEffectiveLevel() <= logging.WARNING:
            print("Receiving {}".format(module["root"]))
        
        # WARNING: make sure there is no "/" at the end of the module
        module["root"] = module["root"].rstrip("/")
        
        command = [
            "rsync",
            "--archive", "--xattrs", "--delete", "--fake-super"
        ]
        command.extend(get_verbosity_options(progress))
        
        command += get_filters(configuration["filters"])
        command += get_filters(module["filters"])
        command += [
            os.path.join(source, module["root"].lstrip("/"), ""),
            os.path.join(module["root"], "")
        ]
        
        call_subprocess(command, "receive", module)
        
    state.previous_direction = "receive"
    state.save()

def read_configuration(path):
    configuration = {
        "modules": [],
        "filters": []
    }
    
    if os.path.isfile(path):
        with open(path) as fd:
            data = yaml.load(fd)
            if data:
                configuration.update(data)
    
    for module in configuration["modules"]:
        if not module["root"].startswith("/"):
            raise Exception(
                "Module path \"{}\" is not absolute".format(module["root"]))
        module.setdefault("filters", [])
    
    return configuration

def confirm(message):
    user_input = ""
    while user_input.lower() not in ["y", "n"]: 
        user_input = six.moves.input("{} [yn] ".format(message))
    return (user_input == "y")



import logging
import subprocess
import sys

sneakersync = sys.modules["sneakersync"]

def get_filters(filters):
    """Return the rsync options for the given filters."""
    
    arguments = []
    for filter_ in filters:
        if len(filter_) > 1:
            raise Exception(
                "Filter must contain only one entry: {}".format(filter_))
        if "exclude" in filter_:
            arguments += ["--exclude", filter_["exclude"]]
        elif "include" in filter_:
            arguments += ["--include", filter_["include"]]
        else:
            raise Exception(
                "Filter must contain include or exclude: {}".format(filter_))
    return arguments

def get_verbosity_options(progress):
    options = []
    if progress:
        options.append("--progress")
    if sneakersync.logger.getEffectiveLevel() <= logging.INFO:
        options.append("--stats")
    if sneakersync.logger.getEffectiveLevel() <= logging.DEBUG:
        options.extend(["--verbose", "--verbose"])
    return options

def call_subprocess(command, action, module):
    """Call a command, redirect output given current logging level."""
    
    output = []
    process = subprocess.Popen(
        command, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
    for line in iter(process.stdout.readline, b""):
        output.append(line)
        encoding = (
            getattr(sys.stdout, "encoding", None)
            or getattr(sys.stdin, "encoding", None)
            or "utf-8"
        )
        sys.stdout.write(line.decode(encoding))
    
    process.wait()
    process.stdout.close()
    if process.returncode != 0:
        raise sneakersync.Exception(action, module, "".join(output))



from codecs import open
import glob
import os

from setuptools import setup, find_packages

here = os.path.dirname(os.path.abspath(__file__))
with open(os.path.join(here, "README"), encoding="utf-8") as f:
    long_description = f.read()

setup(
    name="sneakersync",
    version="1.1.1",
    
    description="Synchronize files through the sneakernet",
    long_description=long_description,
    
    url="https://github.com/lamyj/sneakersync",
    
    author="Julien Lamy",
    author_email="julien@seasofcheese.net",
    
    license="MIT",
    
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        
        "Environment :: Console",
        
        "Intended Audience :: End Users/Desktop",
        "Intended Audience :: Information Technology",
        "Intended Audience :: System Administrators",
        
        "Topic :: Communications :: File Sharing",
        "Topic :: System :: Archiving :: Mirroring",
        
        "License :: OSI Approved :: MIT License",
        
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 3",
    ],
    
    keywords="synchronization, removable drive, sneakernet, rsync",

    packages=find_packages(),
    include_package_data=True,
    install_requires=["pyyaml", "six"],
    
    entry_points={ "console_scripts": [ "sneakersync=sneakersync.main:main"] },
)



import copy
import os
import yaml

class State(object):
    """State of the sneakernet (previous operation, previous host, etc.)."""
    
    def __init__(self, path, previous_direction, previous_date, previous_host):
        self.path = path
        self.previous_direction = previous_direction
        self.previous_date = previous_date
        self.previous_host = previous_host
    
    def save(self):
        data = copy.copy(vars(self))
        del data["path"]
        with open(self.path, "w") as fd:
            yaml.dump(data, fd)
    
    @staticmethod
    def load(path):
        state = {
            "previous_direction": None,
            "previous_date": None,
            "previous_host": None,
        }
        
        if os.path.isfile(path):
            with open(path) as fd:
                data = yaml.load(fd)
                if data:
                    state.update(data)
        
        return State(path, **state)
